import { Firestore, CollectionReference, DocumentReference, DocumentSnapshot, Query, WhereFilterOp } from '@google-cloud/firestore';
import * as E from 'fp-ts/lib/Either';
import * as R from 'fp-ts/lib/Reader';
import * as RE from 'fp-ts/lib/ReaderEither';
import * as RT from 'fp-ts/lib/ReaderTask';
import * as RTE from 'fp-ts/lib/ReaderTaskEither';
import * as T from 'fp-ts/lib/Task';
import * as TE from 'fp-ts/lib/TaskEither';
import QuerySnapshot = FirebaseFirestore.QuerySnapshot;
import QueryDocumentSnapshot = FirebaseFirestore.QueryDocumentSnapshot;
export interface Model {
    id: string;
}
export declare type Operation = WhereFilterOp;
export declare type Condition = [string, Operation, unknown];
/**
 * ```haskell
 * isModel :: a -> bool
 * ```
 */
export declare const isModel: (a: unknown) => a is Model;
/**
 * ```haskell
 * getCollectionFromFirestore :: Firestore -> Reader String Collection
 * ```
 */
export declare const getCollectionFromFirestore: (firestore: Firestore) => R.Reader<string, CollectionReference>;
/**
 * ```haskell
 * getDocumentFromCollection :: Collection -> Reader Model Document
 * ```
 */
export declare const getDocumentFromCollection: (collection: CollectionReference) => R.Reader<Model, DocumentReference>;
/**
 * ```haskell
 * getQueryFromCollection :: Collection -> Reader [Condition] Document Error
 * ```
 */
export declare const getQueryFromCollection: (collection: CollectionReference) => RE.ReaderEither<Condition[], Error, Query>;
/**
 * ```haskell
 * getCollectionFromDocument :: Document -> Reader String Collection
 * ```
 */
export declare const getCollectionFromDocument: (document: DocumentReference) => R.Reader<string, CollectionReference>;
/**
 * ```haskell
 * storeModelToDocument :: Document -> ReaderTaskEither Model Model Error
 * ```
 */
export declare const storeModelToDocument: (document: DocumentReference) => RTE.ReaderTaskEither<Model, Error, Model>;
/**
 * ```haskell
 * storeModelToCollection :: Collection -> String -> ReaderTaskEither Model Model Error
 * ```
 */
export declare const storeModelToCollection: (collection: CollectionReference) => RTE.ReaderTaskEither<Model, Error, Model>;
/**
 * ```haskell
 * storeModelToFirestore :: Firestore -> Reader String (ReaderTaskEither Model Model Error)
 * ```
 */
export declare const storeModelToFirestore: (firestore: Firestore) => R.Reader<string, RTE.ReaderTaskEither<Model, Error, Model>>;
/**
 * ```haskell
 * storeModelToFirestoreWith :: Firestore -> Reader String (ReaderTaskEither (() -> Model) Model Error)
 * ```
 */
export declare const storeModelToFirestoreWith: <A>(firestore: Firestore) => R.Reader<string, R.Reader<(a: A) => Model, RTE.ReaderTaskEither<A, Error, Model>>>;
/**
 * ```haskell
 * updateModelInDocumentTask :: Collection -> ReaderTask Model Model
 * ```
 */
export declare const updateModelInDocumentTask: (document: DocumentReference) => RT.ReaderTask<Model, Model>;
/**
 * ```haskell
 * updateModelInDocument :: Collection -> ReaderTaskEither Model Model Error
 * ```
 */
export declare const updateModelInDocument: (document: DocumentReference) => RTE.ReaderTaskEither<Model, Error, Model>;
/**
 * ```haskell
 * getSnapshotFromDocumentTask :: Document -> Task Snapshot
 * ```
 */
export declare const getSnapshotFromDocumentTask: (document: DocumentReference) => T.Task<DocumentSnapshot>;
/**
 * ```haskell
 * getSnapshotFromDocument :: Document -> TaskEither Snapshot Error
 * ```
 */
export declare const getSnapshotFromDocument: (document: DocumentReference) => TE.TaskEither<Error, DocumentSnapshot>;
/**
 * ```haskell
 * validateSnapshotExistence :: snapshot -> Either a Error
 * ```
 */
export declare const validateSnapshotExistence: (snapshot: DocumentSnapshot) => E.Either<Error, DocumentSnapshot>;
/**
 * ```haskell
 * validateModel :: a -> Either Model Error
 * ```
 */
export declare const validateModel: (a: unknown) => E.Either<Error, Model>;
/**
 * ```haskell
 * getModelFromCollection :: Collection -> ReaderTaskEither Model Model Error
 * ```
 */
export declare const getModelFromCollection: (collection: CollectionReference) => RTE.ReaderTaskEither<Model, Error, Model>;
/**
 * ```haskell
 * getModelFromFirestore :: Firestore -> Reader String (ReaderTaskEither Model Model Error)
 * ```
 */
export declare const getModelFromFirestore: (firestore: Firestore) => R.Reader<string, RTE.ReaderTaskEither<Model, Error, Model>>;
/**
 * ```haskell
 * listCollectionsInFirestore :: Firestore -> Task [Collection]
 * ```
 */
export declare const listCollectionsInFirestore: (firestore: Firestore) => T.Task<CollectionReference[]>;
/**
 * ```haskell
 * listDocumentsInCollection :: Collection -> Task [Document]
 * ```
 */
export declare const listDocumentsInCollection: (collection: CollectionReference) => T.Task<DocumentReference[]>;
/**
 * ```haskell
 * getModelFromDocument :: Document -> TaskEither Model Error
 * ```
 */
export declare const getModelFromDocument: (document: DocumentReference) => TE.TaskEither<Error, Model>;
/**
 * ```haskell
 * getSnapshotFromQueryTask :: Query -> Task Snapshot
 * ```
 */
export declare const getSnapshotFromQueryTask: (query: Query) => T.Task<QuerySnapshot>;
/**
 * ```haskell
 * getSnapshotFromQuery :: Query -> TaskEither Snapshot Error
 * ```
 */
export declare const getSnapshotFromQuery: (query: Query) => TE.TaskEither<Error, QuerySnapshot>;
/**
 * ```haskell
 * getDocumentSnapshotsFromQuerySnapshot :: QuerySnapshot -> [DocumentSnapshot]
 * ```
 */
export declare const getDocumentSnapshotsFromQuerySnapshot: (snapshot: QuerySnapshot) => QueryDocumentSnapshot[];
/**
 * ```haskell
 * getModelsFromQuerySnapshot :: QuerySnapshot -> Either [Model] Error
 * ```
 */
export declare const getModelsFromQuerySnapshot: (snapshot: QuerySnapshot) => E.Either<Error, Model[]>;
/**
 * ```haskell
 * getModelsFromQuery :: Query -> TaskEither [Model] Error
 * ```
 */
export declare const getModelsFromQuery: (query: Query) => TE.TaskEither<Error, Model[]>;
/**
 * ```haskell
 * getModelsInCollectionWhere :: Collection -> ReaderTaskEither [Condition] [Model] Error
 * ```
 */
export declare const getModelsInCollectionWhere: (collection: CollectionReference) => RTE.ReaderTaskEither<Condition[], Error, Model[]>;
/**
 * ```haskell
 * getModelsInFirestoreWhere :: Firestore -> Reader String (ReaderTaskEither [Condition] [Model] Error)
 * ```
 */
export declare const getModelsInFirestoreWhere: (firestore: Firestore) => R.Reader<string, RTE.ReaderTaskEither<Condition[], Error, Model[]>>;
/**
 * ```haskell
 * getModelsInCollection :: Collection -> TaskEither [Model] Error
 * ```
 */
export declare const getModelsInCollection: (collection: CollectionReference) => TE.TaskEither<Error, Model[]>;
/**
 * ```haskell
 * getModelsInFirestore :: Firestore -> ReaderTaskEither String [Model] Error
 * ```
 */
export declare const getModelsInFirestore: (firestore: Firestore) => RTE.ReaderTaskEither<string, Error, Model[]>;
